@@ -19,11 +19,11 @@
 extern crate test;
 
 use bitcoin::BlockHash;
-use log::{info, Level};
+use log::{error, info, Level};
 use std::fs::File;
 
 use std::path::PathBuf;
-use std::sync::mpsc::{sync_channel, SyncSender};
+use std::sync::mpsc::{sync_channel, Receiver, SyncSender};
 use std::sync::{Arc, Mutex};
 use std::thread;
 use std::thread::JoinHandle;
@@ -31,8 +31,11 @@ use std::time::Instant;
 use structopt::StructOpt;
 use utxo::AnyUtxo;
 
+pub use period::{PeriodCounter, Periodic};
+
 mod block_extra;
 mod fee;
+mod period;
 mod pipe;
 mod read_detect;
 mod reorder;
@@ -135,11 +138,8 @@ pub fn iterate(config: Config, channel: SyncSender<Option<BlockExtra>>) -> JoinH
 
         // FsBlock is a small struct (~120b), so 10_000 is not a problem but allows the read_detect to read ahead the next block file
         let (send_block_fs, receive_block_fs) = sync_channel(0);
-        let mut read =
+        let _read =
             read_detect::ReadDetect::new(config.blocks_dir.clone(), config.network, send_block_fs);
-        let read_handle = thread::spawn(move || {
-            read.start();
-        });
 
         let (send_ordered_blocks, receive_ordered_blocks) =
             sync_channel(config.channels_size.into());
@@ -149,31 +149,61 @@ pub fn iterate(config: Config, channel: SyncSender<Option<BlockExtra>>) -> JoinH
         } else {
             send_ordered_blocks
         };
-        let mut reorder = reorder::Reorder::new(
+        let _reorder = reorder::Reorder::new(
             config.network,
             config.max_reorg,
             receive_block_fs,
             send_ordered_blocks,
         );
-        let orderer_handle = thread::spawn(move || {
-            reorder.start();
-        });
 
         if !config.skip_prevout {
-            let mut fee = fee::Fee::new(receive_ordered_blocks, channel, config.utxo_manager());
-            let fee_handle = thread::spawn(move || {
-                fee.start();
-            });
-            fee_handle.join().unwrap();
+            let _fee = fee::Fee::new(receive_ordered_blocks, channel, config.utxo_manager());
         }
 
-        read_handle.join().unwrap();
-        orderer_handle.join().unwrap();
         info!("Total time elapsed: {}s", now.elapsed().as_secs());
     })
 }
 
+struct BlockExtraIterator {
+    handle: Option<JoinHandle<()>>,
+    recv: Receiver<Option<BlockExtra>>,
+}
+impl Iterator for BlockExtraIterator {
+    type Item = BlockExtra;
+
+    fn next(&mut self) -> Option<Self::Item> {
+        match self.recv.recv() {
+            Ok(Some(val)) => Some(val),
+            Ok(None) => {
+                if let Some(handle) = self.handle.take() {
+                    handle.join().unwrap();
+                }
+                None
+            }
+            Err(e) => {
+                error!("error iterating {:?}", e);
+                if let Some(handle) = self.handle.take() {
+                    handle.join().unwrap();
+                }
+                None
+            }
+        }
+    }
+}
+
+/// Return an Iterator of [`BlockExtra`] read from `blocks*.dat` contained in the `config.blocks_dir`
+/// Blocks returned are iterated in order, starting from the genesis to the highest block
+/// (minus `config.max_reorg`) in the directory, unless `config.stop_at_height` is specified.
+pub fn iter(config: Config) -> impl Iterator<Item = BlockExtra> {
+    let (send, recv) = sync_channel(config.channels_size.into());
+
+    let handle = Some(iterate(config, send));
+
+    BlockExtraIterator { handle, recv }
+}
+
 /// Utility method usually returning [log::Level::Debug] but when `i` is divisible by `every` returns [log::Level::Info]
+#[deprecated]
 pub fn periodic_log_level(i: u32, every: u32) -> Level {
     if i % every == 0 {
         Level::Info
@@ -320,6 +350,28 @@ mod bench {
         });
     }
 
+    #[bench]
+    fn bench_bitcoin_hashes_sha_long(b: &mut Bencher) {
+        let a: Vec<_> = (0u8..255).cycle().take(1000).collect();
+        b.iter(|| {
+            let mut engine = sha256::Hash::engine();
+            engine.input(&a);
+            let hash = sha256::Hash::from_engine(engine);
+            black_box(hash);
+        });
+    }
+
+    #[bench]
+    fn bench_sha2_crate_long(b: &mut Bencher) {
+        let a: Vec<_> = (0u8..255).cycle().take(1000).collect();
+        b.iter(|| {
+            let mut hasher = Sha256::new();
+            hasher.update(&a);
+            let hash = hasher.finalize();
+            black_box(hash);
+        });
+    }
+
     #[bench]
     fn bench_fxhash(b: &mut Bencher) {
         let outpoint = OutPoint::default();
