@@ -1,8 +1,8 @@
 use crate::bitcoin::consensus::Decodable;
-use crate::bitcoin::{Block, BlockHash, Network};
-use crate::{periodic_log_level, FsBlock};
-use bitcoin::Error;
-use log::{error, info, log};
+use crate::bitcoin::{BlockHash, Network};
+use crate::{FsBlock, Periodic};
+use bitcoin::{BlockHeader, Error, Transaction, VarInt};
+use log::{error, info};
 use std::collections::HashSet;
 use std::convert::TryInto;
 use std::fs::File;
@@ -10,13 +10,19 @@ use std::io::{BufReader, Read, Seek};
 use std::path::PathBuf;
 use std::sync::mpsc::SyncSender;
 use std::sync::{Arc, Mutex};
-use std::time::Instant;
+use std::thread::JoinHandle;
+use std::time::{Duration, Instant};
 
 pub struct ReadDetect {
-    blocks_dir: PathBuf,
-    seen: Seen,
-    network: Network,
-    sender: SyncSender<Option<Vec<FsBlock>>>,
+    join: Option<JoinHandle<()>>,
+}
+
+impl Drop for ReadDetect {
+    fn drop(&mut self) {
+        if let Some(jh) = self.join.take() {
+            jh.join().expect("thread failed");
+        }
+    }
 }
 
 /// Save half memory in comparison to using directly HashSet<BlockHash> while providing enough
@@ -58,61 +64,54 @@ impl ReadDetect {
         network: Network,
         sender: SyncSender<Option<Vec<FsBlock>>>,
     ) -> Self {
-        ReadDetect {
-            blocks_dir,
-            sender,
-            seen: Seen::new(),
-            network,
+        let mut periodic = Periodic::new(Duration::from_secs(60));
+        Self {
+            join: Some(std::thread::spawn(move || {
+                let mut now = Instant::now();
+                let mut seen = Seen::new();
+                let mut path = blocks_dir.clone();
+                path.push("blk*.dat");
+                info!("listing block files at {:?}", path);
+                let mut paths: Vec<PathBuf> = glob::glob(path.to_str().unwrap())
+                    .unwrap()
+                    .map(|r| r.unwrap())
+                    .collect();
+                paths.sort();
+                info!("There are {} block files", paths.len());
+                let mut busy_time = 0u128;
+
+                for path in paths.into_iter() {
+                    let file = File::open(&path).unwrap();
+                    let mut reader = BufReader::new(file);
+                    let detected_blocks = detect(&mut reader, network.magic()).unwrap();
+                    drop(reader);
+
+                    let file = File::open(&path).unwrap();
+                    let file = Arc::new(Mutex::new(file));
+
+                    let fs_blocks: Vec<_> = detected_blocks
+                        .into_iter()
+                        .filter(|e| seen.insert(&e.hash))
+                        .map(|e| e.into_fs_block(&file))
+                        .collect();
+
+                    // TODO if 0 blocks found, maybe wrong directory
+                    if periodic.elapsed() {
+                        info!("read {:?}, contains {} blocks", &path, fs_blocks.len());
+                    }
+
+                    busy_time += now.elapsed().as_nanos();
+                    sender.send(Some(fs_blocks)).expect("cannot send");
+                    now = Instant::now();
+                }
+                info!(
+                    "ending reader parse , busy time: {}s",
+                    (busy_time / 1_000_000_000)
+                );
+                sender.send(None).expect("cannot send");
+            })),
         }
     }
-
-    pub fn start(&mut self) {
-        let mut now = Instant::now();
-        let mut path = self.blocks_dir.clone();
-        path.push("blk*.dat");
-        info!("listing block files at {:?}", path);
-        let mut paths: Vec<PathBuf> = glob::glob(path.to_str().unwrap())
-            .unwrap()
-            .map(|r| r.unwrap())
-            .collect();
-        paths.sort();
-        info!("There are {} block files", paths.len());
-        let mut busy_time = 0u128;
-
-        for (count, path) in paths.into_iter().enumerate() {
-            let file = File::open(&path).unwrap();
-            let mut reader = BufReader::new(file);
-            let detected_blocks = detect(&mut reader, self.network.magic()).unwrap();
-            drop(reader);
-
-            let file = File::open(&path).unwrap();
-            let file = Arc::new(Mutex::new(file));
-
-            let fs_blocks: Vec<_> = detected_blocks
-                .into_iter()
-                .filter(|e| self.seen.insert(&e.hash))
-                .map(|e| e.into_fs_block(&file))
-                .collect();
-
-            // TODO if 0 blocks found, maybe wrong directory
-
-            log!(
-                periodic_log_level(count as u32, 100),
-                "read {:?}, contains {} blocks",
-                &path,
-                fs_blocks.len()
-            );
-
-            busy_time += now.elapsed().as_nanos();
-            self.sender.send(Some(fs_blocks)).expect("cannot send");
-            now = Instant::now();
-        }
-        info!(
-            "ending reader parse , busy time: {}s",
-            (busy_time / 1_000_000_000)
-        );
-        self.sender.send(None).expect("cannot send");
-    }
 }
 
 pub fn detect<R: Read + Seek>(mut reader: &mut R, magic: u32) -> Result<Vec<DetectedBlock>, Error> {
@@ -135,16 +134,29 @@ pub fn detect<R: Read + Seek>(mut reader: &mut R, magic: u32) -> Result<Vec<Dete
         };
         let size = u32::consensus_decode(&mut reader)?;
         let start = reader.stream_position().unwrap() as usize;
-        match Block::consensus_decode(&mut reader) {
-            Ok(block) => {
+        match BlockHeader::consensus_decode(&mut reader) {
+            Ok(block_header) => {
+                // Instead of parsing a block which is unneeded at this stage
+                // we just seek over transactions to avoid full Block allocation
+                let n_txs = match VarInt::consensus_decode(&mut reader) {
+                    Ok(v) => v.0,
+                    Err(_) => continue,
+                };
+                for _ in 0..n_txs {
+                    if Transaction::consensus_decode(&mut reader).is_err() {
+                        continue;
+                    }
+                }
                 let end = reader.stream_position().unwrap() as usize;
-                assert_eq!(size as usize, end - start);
-                let hash = block.header.block_hash();
+                if size as usize != end - start {
+                    continue;
+                }
+                let hash = block_header.block_hash();
                 let detected_block = DetectedBlock {
                     start,
                     end,
                     hash,
-                    prev: block.header.prev_blockhash,
+                    prev: block_header.prev_blockhash,
                 };
                 detected_blocks.push(detected_block);
             }
@@ -152,7 +164,7 @@ pub fn detect<R: Read + Seek>(mut reader: &mut R, magic: u32) -> Result<Vec<Dete
                 // It's mandatory to use stream_position (require MSRV 1.51) because I can't maintain
                 // a byte read position because in case of error I don't know how many bytes of the
                 // reader has been consumed
-                error!("error block parsing {:?}", e)
+                error!("error block header parsing {:?}", e)
             }
         }
     }
